%=============================================================================
% Copyright (c) 2016-present Allan CORNET (Nelson)
%=============================================================================
% This file is part of the Nelson.
%=============================================================================
% LICENCE_BLOCK_BEGIN
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU Lesser General Public
% License as published by the Free Software Foundation; either
% version 2.1 of the License, or (at your option) any later version.
%
% Alternatively, you can redistribute it and/or
% modify it under the terms of the GNU General Public License as
% published by the Free Software Foundation; either version 2 of
% the License, or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU Lesser General Public License for more details.
%
% You should have received a copy of the GNU Lesser General Public
% License along with this program. If not, see <http://www.gnu.org/licenses/>.
% LICENCE_BLOCK_END
%=============================================================================
assert_isequal(nargin('cosm'), 1);
assert_isequal(nargout('cosm'), 1);
%=============================================================================
assert_isequal(cosm(NaN), NaN);
assert_isequal(cosm(-NaN), NaN);
assert_isequal(cosm(Inf), NaN);
assert_isequal(cosm(-Inf), NaN);
%=============================================================================
x = cosm(0i);
ref = 1;
assert_isequal(x, ref);
assert_isequal(cos(0), 1);
assert_isequal(cos(-0), 1);
%=============================================================================
X = cosm(zeros(3, 3));
REF = eye(3, 3);
assert_isequal(X, REF);
%=============================================================================
assert_isequal(cosm([]), []);
%=============================================================================
A = [1,2;3,4];
R = cosm(A);
REF = [0.8554    -0.1109;
-0.1663     0.6891];
assert_isapprox(R, REF, 1e-4);
%=============================================================================
A = single([1,2;3,4]);
R = cosm(A);
REF = [0.8554    -0.1109;
-0.1663     0.6891];
assert_isapprox(R, REF, 1e-4);
assert_isequal(class(R), 'single');
%=============================================================================
A = [1,2;3,4] + i;
R = cosm(A);
REF = [1.0181+0.7790i, 0.2173+1.0167i;
0.5076+1.3799i, 1.8889+1.8687i];
assert_isapprox(R, REF, 1e-4);
%=============================================================================
A = single([1,2;3,4] + i);
R = cosm(A);
REF = [1.0181+0.7790i, 0.2173+1.0167i;
0.5076+1.3799i, 1.8889+1.8687i];
assert_isapprox(R, REF, 1e-4);
assert_isequal(class(R), 'single');
%=============================================================================
assert_checkerror('cosm([1 , 2])', _('Square matrix expected.'));
assert_checkerror('cosm(''a'')', [_('Undefined function ''cosm'' for input arguments of type '''), class('a'), '''.']);
%=============================================================================
